/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import QtQuick.XmlListModel 2.0
import "ClockScript.js" as Cscript
import "../common/Constants.js" as Constants
import "../common/ClockUtils.js" as Utils
import "../common"

AnalogFaceBase {
    id: clockRoot

    property var currDate: new Date;
    property int hours: currDate.getHours()
    property int minutes: currDate.getMinutes()
    property int seconds: currDate.getUTCSeconds();
    property alias hourHandRect: hourHand
    property alias minuteHandRect: minuteHand
    property alias secondsHandRect: secondsHand
    property real latitude: 52.01;
    property real longitude: 4.35;

    function timeChanged(now) {
        hours = now.getHours()
        minutes = now.getMinutes()
        seconds = now.getUTCSeconds();
    }

    function reloadSunRiseModel () {
        sunRiseModel.reload()
    }

    // TODO: Get the latitude and longitude using the platform API automatically instead of using static values.
    XmlListModel {
        id: sunRiseModel

        source: Cscript.getUrlString(latitude, longitude)
        query: "/current/city"

        XmlRole { name: "sunrise"; query: "sun/@rise/string()"; isKey: true }
        XmlRole { name: "sunset"; query: "sun/@set/string()"; isKey: true  }
    }

    AnalogClockHand {
        id: hourHand
        height: units.gu(8)
        // TODO: set source according to theme?
        rotationAngle: (clockRoot.hours * 30) + (clockRoot.minutes / 2)
    }

    AnalogClockHand {
        id: minuteHand
        height: units.gu(12)
        // TODO: set source according to theme?
        rotationAngle: clockRoot.minutes * 6
    }

    AnalogClockHand {
        id: secondsHand
        // TODO: set source according to theme?
        height: units.gu(14); width:  units.gu(0.5)
        rotationAngle: clockRoot.seconds * 6
    }

    Repeater {
        model: ["1", "2", "4", "5", "7", "8", "10", "11"]
        delegate: AnalogClockMarker {
            number: modelData
            size:0.7
        }
    }

    Repeater {
        model: ["3", "6", "9", "12"]
        delegate: AnalogClockMarker {
            number: modelData
            size: 1
            color: Constants.pitchBlack
        }
    }

    MouseArea {
        anchors.fill: parent
        onClicked: {
            // Only show the sunrise/sunset times after the XmlListModel is Ready.
            if (sunRiseModel.status == 1) {
                clockRoot.state == "" ? clockRoot.state = "easteregg" : clockRoot.state = "";
            }
            else {
                Utils.log("Sunrise/Sunset times not yet loaded.")
            }
        }
    }

    Rectangle {
        id: easterEggCircle;

        width: parent.width; height: parent.height
        radius: parent.radius
        z: parent.z + 100
        antialiasing: parent.antialiasing
        anchors.centerIn: parent
        color: Constants.brightWhite
        opacity: 0
        visible: opacity == 0 ? false : true

        Column {
            spacing: units.gu(1)
            anchors.centerIn: parent
            width: parent.width

            Label {
                id: sunriseLabel
                anchors.horizontalCenter: parent.horizontalCenter

                fontSize: "large"
                text: i18n.tr("Sunrise")
            }

            Label {
                id: sunriseTimeLabel
                anchors.horizontalCenter: parent.horizontalCenter

                fontSize: "x-large"
                text: i18n.tr("loading")
            }

            Rectangle {
                id: sunsetsunriseDivider
                width: parent.width - units.gu(12)
                height: units.gu(1) / 2
                anchors.horizontalCenter: parent.horizontalCenter

                color: Constants.normalGrey
            }

            Label {
                id: sunsetTimeLabel
                width: parent.width
                horizontalAlignment: Text.AlignHCenter

                fontSize: "x-large"
                text: i18n.tr("loading")
            }

            Label {
                id: sunsetLabel
                width: parent.width
                horizontalAlignment: Text.AlignHCenter

                fontSize: "large"
                text: i18n.tr("Sunset")
            }

        }
    }

    states: [
        State {
            name: "easteregg"            
            PropertyChanges { target: easterEggCircle; opacity: 1; }
            // TODO: Do some caching of data here. Store data locally to reduce API calls.
            PropertyChanges { target: sunriseTimeLabel; text: Cscript.getSunTime(sunRiseModel.get(0).sunrise); }
            PropertyChanges { target: sunsetTimeLabel; text: Cscript.getSunTime(sunRiseModel.get(0).sunset); }
        },
        State {
            name: ""            
            PropertyChanges { target: easterEggCircle; opacity: 0; }
        }

    ]
}
