/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common"
import "../common/Constants.js" as Constants
import "../common/ClockUtils.js" as Utils
import "TimerScript.js" as TScript

Page {
    id: timerPage

    property int lastindex: -1;
    property string selectItemPreset: "";

    property alias hours: analogTimer.hours;
    property alias minutes: analogTimer.minutes;
    property alias seconds: analogTimer.seconds;
    property alias totalTime: analogTimer.totalTime;
    property alias timerValue: analogTimer.timerValue;
    property alias timerOn: analogTimer.timerOn;

    function onTimerUpdate() {
        analogTimer.onTimerUpdate();
    }

    function reset() {
        analogTimer.reset();
        namePreset.text = "";
    }

    Component.onCompleted: {
        Utils.log("TimerPage loaded");
        TScript.populatePreset();
        Utils.log("Saved Timer Presets Populated")
    }

    anchors.margins: units.gu(2)

    states: [
        State {
            name: "ADDPRESET"
            PropertyChanges { target: addPresetCircleDone; visible: true }
            PropertyChanges { target: namePreset; visible: true }
        }
    ]

    TimerStorage {
        id: storageTimer
    }

    // Timer label hh:mm:ss
    TimerLabelRow {
        id: topTimerRow

        state: "FOCUS-SEC"
        timerOn: analogTimer.timerOn;
        hoursLabel: Utils.zeroleft(hours, 2);
        minutesLabel: Utils.zeroleft(minutes, 2);
        secondsLabel: Utils.zeroleft(seconds, 2);

        states:[
            State {
                name: "FOCUS-SEC"
                PropertyChanges { target: topTimerRow; secondsLabelFlag: true }
                PropertyChanges { target: topTimerRow; minutesLabelFlag: false }
                PropertyChanges { target: topTimerRow; hoursLabelFlag: false }
                PropertyChanges { target: analogTimer; timerHandRotation: seconds * 6 }
            },

            State {
                name: "FOCUS-MIN"
                PropertyChanges { target: topTimerRow; secondsLabelFlag: false }
                PropertyChanges { target: topTimerRow; minutesLabelFlag: true }
                PropertyChanges { target: topTimerRow; hoursLabelFlag: false }
                PropertyChanges { target: analogTimer; timerHandRotation: minutes * 6 }
            },

            State {
                name: "FOCUS-HOR"
                PropertyChanges { target: topTimerRow; secondsLabelFlag: false }
                PropertyChanges { target: topTimerRow; minutesLabelFlag: false }
                PropertyChanges { target: topTimerRow; hoursLabelFlag: true }
                PropertyChanges { target: analogTimer; timerHandRotation: hours * 6 }
            },

            State {
                name: "DISABLED"
                when: timerOn == true || timerAnimationContainer.state == "SCROLLED" || buttonTimer.state == "DONE"
                PropertyChanges { target: topTimerRow; secondsLabelFlag: false }
                PropertyChanges { target: topTimerRow; minutesLabelFlag: false }
                PropertyChanges { target: topTimerRow; hoursLabelFlag: false }
            }
        ]
    }

    AnimationContainer {
        id: timerAnimationContainer

        // FIXME: The y is given a arbitary value which needs to be fixed by ensuring that the outerglow is not drawn outside the clock face area.
        initYPos: units.gu(8);
        finalYPos: -listPreset.height

        anchors { left: parent.left; top: topTimerRow.bottom; right: parent.right; bottom: parent.bottom }

        // Main element which draws the timer face and manages time.
        AnalogTimer {
            id: analogTimer

            anchors { top: parent.top; horizontalCenter: parent.horizontalCenter }

            secondsFlag: topTimerRow.secondsLabelFlag;
            minutesFlag: topTimerRow.minutesLabelFlag;
            hoursFlag: topTimerRow.hoursLabelFlag;
            inProgressFlag: (timerOn == true || buttonTimer.state == "DONE") ? true : false;

            onHoursChanged: topTimerRow.hoursLabel = Utils.zeroleft(hours, 2);
            onMinutesChanged: topTimerRow.minutesLabel = Utils.zeroleft(minutes, 2);
            onSecondsChanged: topTimerRow.secondsLabel = Utils.zeroleft(seconds, 2);
            onTimerOnChanged: topTimerRow.timerOn = timerOn;
        }

        // Multi-function button (Preset, Start, Stop, Done)
        Button {
            id: buttonTimer

            width: units.gu(12); height: units.gu(5)
            anchors { top: analogTimer.bottom; topMargin: units.gu(7); horizontalCenter: parent.horizontalCenter }
            color: Constants.brightWhite
            visible: timerPage.state == "ADDPRESET" ? false : true;

            text: i18n.tr("Presets")

            states:[
                State {
                    name: "START"
                    when: ((seconds != 0 || minutes != 0 || hours != 0) && timerOn == false)
                    PropertyChanges { target: buttonTimer; color: Constants.ubuntuOrange; text: i18n.tr("Start") }
                    PropertyChanges { target: analogTimer; timerHandColor: Constants.ubuntuOrange }
                    PropertyChanges { target: analogTimer; glowColor: Constants.analogClockFaceGlowColor }
                },

                State {
                    name: "STOP"
                    PropertyChanges { target: buttonTimer; color: Constants.darkGrey; text: i18n.tr("Stop") }
                    PropertyChanges { target: analogTimer; timerHandColor: Constants.ubuntuOrange }
                    PropertyChanges { target: analogTimer; glowColor: Constants.analogClockFaceGlowColor }
                },

                State {
                    name: "DONE"
                    when: (totalTime == -1 && timerOn == false)
                    PropertyChanges { target: buttonTimer; color: Constants.green; text: i18n.tr("Done") }
                    PropertyChanges { target: analogTimer; timerHandColor: Constants.green }
                    PropertyChanges { target: analogTimer; glowColor: Constants.analogClockFaceGlowColorTimerDown }
                }
            ]

            onClicked: {
                switch (buttonTimer.state) {
                case "":
                    timerAnimationContainer.swipeView()
                    break;

                case "START":
                    totalTime = TScript.totalSeconds(hours, minutes, seconds);
                    timerOn = true;
                    state = "STOP";

                    if (timerAnimationContainer.state == "SCROLLED") {
                        timerAnimationContainer.swipeView()
                    }
                    break;

                case "STOP":
                    timerOn = false;
                    reset();
                    break;

                case "DONE":
                    reset();
                    break;
                }
            }
        }

        // Column element to hold the saved presets
        Column {
            id: listPreset

            height: childrenRect.height
            visible: timerPage.state == "ADDPRESET" ? false : true;
            anchors { left: parent.left; right: parent.right; top: buttonTimer.bottom; topMargin: units.gu(5) }

            ListItem.Divider { }

            ListItem.Header { text: i18n.tr("Saved Presets") }

            ListView {
                id: listTimerPreset

                anchors { left: parent.left; right: parent.right }
                height: units.gu(35)
                model: listModel

                delegate: ListItem.Standard {
                    text: nametimer

                    Label {
                        fontSize: "medium"
                        text: TScript.getstringTimer(secondstimer)
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                    }

                    selected: select
                    removable: true

                    backgroundIndicator: SwipeDelete {
                        id: swipeDelete
                        state: swipingState
                    }

                    // TODO: Implement different based actions based on the swipe direction.
                    onItemRemoved: {
                        storageTimer.delTimerValue(text);
                        TScript.populatePreset();
                    }

                    onClicked: {
                        if (!timerOn) {
                            buttonTimer.state = "START";
                            analogTimer.setTime(secondstimer);
                            if (lastindex == -1) {
                                listModel.setProperty(index, "select", true)
                                lastindex = index;
                            } else if (lastindex != index) {
                                listModel.setProperty(lastindex, "select", false)
                                listModel.setProperty(index, "select", true)
                                lastindex = index;
                            }
                            selectItemPreset = text;
                        }
                    }
                }
            }

            ListModel { id: listModel }

            WorkerScript {
                id: worker
                source: "TimerScript.js"
            }
        }

        // Add preset done circle which is shown when the add preset toolbar button is pressed
        Rectangle {
            id: addPresetCircleDone

            width: units.gu(12); height: units.gu(12)
            radius: width / 2
            anchors.centerIn: analogTimer
            border.color: Constants.normalGrey
            color: "#F8F8F8"
            antialiasing: true
            visible: false

            Label {
                id: addPresetTextDone

                anchors.centerIn: parent
                color: Constants.green
                fontSize: "large"

                text: i18n.tr("DONE")
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (namePreset.text != "" && (hours > 0 || minutes > 0 || seconds > 0)) {
                        storageTimer.setTimerValue(namePreset.text, TScript.totalSeconds(hours, minutes, seconds));
                        TScript.populatePreset();
                        reset();
                        timerPage.state = ""
                    }
                }
            }
        }

        // Element to set the name of a saved preset.
        TextField {
            id: namePreset

            visible: false
            hasClearButton: true
            anchors { left: analogTimer.left; right: analogTimer.right; top: analogTimer.bottom; topMargin: units.gu(7)}

            placeholderText: i18n.tr("Timer One")
        }
    }

    // Various toolbar actions such as add preset, edit and back.
    tools: ToolbarActions {
        id: toolbarTimer

        Action {
            id: addPreset
            objectName: "add-preset"

            visible: timerPage.state == "ADDPRESET" ? false : true;
            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            text: i18n.tr("Add Preset")

            onTriggered: {
                reset()
                if (timerAnimationContainer.state == "SCROLLED") {
                    timerAnimationContainer.swipeView()
                }

                timerPage.state = "ADDPRESET"

                toolbarTimer.locked = false;
                toolbarTimer.opened = false;
            }
        }

        Action {
            id: editPreset
            objectName: "edit-preset"

            visible: timerPage.state == "ADDPRESET" ? false : true;
            iconSource: Qt.resolvedUrl("../images/add.png")
            text: i18n.tr("Edit")

            onTriggered: {
                timerAnimationContainer.swipeView()

                toolbarTimer.locked = true;
                toolbarTimer.opened = true;
            }
        }

        Action {
            id: deletePreset

            iconSource: Qt.resolvedUrl("../images/remove_icon.png")
            text: i18n.tr("Delete")
            visible: timerAnimationContainer.state == "SCROLLED" ? true : false;

            onTriggered: {
                if (!timerOn) {
                    if (selectItemPreset != "") {
                        storageTimer.delTimerValue(selectItemPreset);
                        TScript.populatePreset();
                    }
                    analogTimer.reset();
                }
            }
        }

        back {
            visible: timerAnimationContainer.state != "DEFAULT" || timerPage.state == "ADDPRESET" ? true : false

            onTriggered: {
                toolbarTimer.locked = false
                toolbarTimer.opened = false

                if (timerAnimationContainer.state == "SCROLLED") {
                    timerAnimationContainer.swipeView()
                }
                else if (timerPage.state == "ADDPRESET") {
                    timerPage.state = ""
                    reset()
                }
            }
        }
    }
}
